(********************************************************************
 * COPYRIGHT -- Bernecker + Rainer
 ********************************************************************
 * PROGRAM: Basic
 * File: basicCyclic.st
 * Author: Bernecker + Rainer
 * Created: December 01, 2009
 ********************************************************************
 * Implementation of Program Basic
 ********************************************************************)
PROGRAM _CYCLIC

(***************************************************************
        Control Sequence
***************************************************************)
	

	//touchProbePosPlcModeDelta:=BasicControlAxis1.Status.TouchProbePosPlcMod-MC_BR_TouchProbe_0.RecordedPeriodicValue;
	
	(* status information is read before the step sequencer to attain a shorter reaction time *)
	BasicControlAxis1.Status.Enabled:=NOT(BasicControlAxis1.AxisState.Disabled);
	BasicControlAxis1.Status.PLCHomeDone:=BasicControlAxis1.Status.DriveStatus.HomingOk;
	BasicControlAxis1.Status.Error:=(BasicControlAxis1.Status.ErrorID<>0) OR (gSdcAxisCtrl.status.errorNumberIO[0]<>0);
	
	typDriveLatchPos.Continuous2:=TRUE;
	typDriveLatchPos.UsePosEdge2:=TRUE;
	DriveTouchProbe;	
	
	typMcBrTriggerRef0.Edge:=mcP_EDGE ;
	typMcBrTriggerRef0.EventSourceParID:=ACP10PAR_STAT_TRIGGER2;//STAT_TRIGGER1,Digital inputs: Status trigger1
	typMcBrTriggerRef0.ProbeParID:=ACP10PAR_PCTRL_S_ACT;//PCTRL_S_ACT,111: CTRL Position controller: Actual position
	MC_BR_TouchProbe_0(
		Axis :=Axis1Obj , 
		TriggerInput :=typMcBrTriggerRef0 , 
		Enable :=bTP0_1,
	Mode:=mcWITHOUT_PERIOD+ mcUSE_AXIS_PERIOD) ;
	
	touchProbePosPlcModeDelta:=MC_ABB_TouchProbe_0.AxisTouchProbePos-MC_BR_TouchProbe_0.RecordedPeriodicValue;
	
(************************ MC_READSTATUS *************************)
	MC_ReadStatus_0.Enable := NOT(MC_ReadStatus_0.Error);
	MC_ReadStatus_0.Axis := Axis1Obj;
	MC_ReadStatus_0();
	BasicControlAxis1.AxisState.Disabled             := MC_ReadStatus_0.Disabled;
	BasicControlAxis1.AxisState.StandStill           := MC_ReadStatus_0.StandStill;
	BasicControlAxis1.AxisState.Stopping             := MC_ReadStatus_0.Stopping;
	BasicControlAxis1.AxisState.Homing               := MC_ReadStatus_0.Homing;
	BasicControlAxis1.AxisState.DiscreteMotion       := MC_ReadStatus_0.DiscreteMotion;
	BasicControlAxis1.AxisState.ContinuousMotion     := MC_ReadStatus_0.ContinuousMotion;
	BasicControlAxis1.AxisState.SynchronizedMotion   := MC_ReadStatus_0.SynchronizedMotion;
	BasicControlAxis1.AxisState.ErrorStop            := MC_ReadStatus_0.Errorstop;

	(********************MC_BR_READDRIVESTATUS***********************)
	MC_BR_ReadDriveStatus_0.Enable := NOT(MC_BR_ReadDriveStatus_0.Error);
	MC_BR_ReadDriveStatus_0.Axis := Axis1Obj;
	MC_BR_ReadDriveStatus_0.AdrDriveStatus := ADR(BasicControlAxis1.Status.DriveStatus);
	MC_BR_ReadDriveStatus_0();

	(******************** MC_READACTUALPOSITION *********************)
	MC_ReadActualPosition_0.Enable := (NOT(MC_ReadActualPosition_0.Error));
	MC_ReadActualPosition_0.Axis := Axis1Obj;
	MC_ReadActualPosition_0();
	IF(MC_ReadActualPosition_0.Valid = TRUE)THEN
		BasicControlAxis1.Status.ActPosition := MC_ReadActualPosition_0.Position;
	END_IF

	(******************** MC_READACTUALVELOCITY *********************)
	MC_ReadActualVelocity_0.Enable := (NOT(MC_ReadActualVelocity_0.Error));
	MC_ReadActualVelocity_0.Axis := Axis1Obj;
	MC_ReadActualVelocity_0();
	IF(MC_ReadActualVelocity_0.Valid = TRUE)THEN
		BasicControlAxis1.Status.ActVelocity := MC_ReadActualVelocity_0.Velocity;
	END_IF

	(************************ MC_READAXISERROR **********************)
	MC_ReadAxisError_0.Enable := NOT(MC_ReadAxisError_0.Error);
	MC_ReadAxisError_0.Axis := Axis1Obj;
	MC_ReadAxisError_0.DataAddress := ADR(BasicControlAxis1.Status.ErrorText);
	MC_ReadAxisError_0.DataLength := SIZEOF(BasicControlAxis1.Status.ErrorText);
	MC_ReadAxisError_0.DataObjectName := 'AcpErrTxt';
	MC_ReadAxisError_0();

	(**************** CHECK FOR GENERAL AXIS ERROR ******************)
	IF ((MC_ReadAxisError_0.AxisErrorID <> 0) AND (MC_ReadAxisError_0.Valid = TRUE)) THEN
		AxisStep := STATE_ERROR_AXIS;
		(***************** CHECK IF POWER SHOULD BE OFF *******************)
	ELSIF ((BasicControlAxis1.Command.Power = FALSE) AND (MC_ReadAxisError_0.Valid = TRUE)) THEN
		IF ((MC_ReadStatus_0.Errorstop = TRUE) AND (MC_ReadStatus_0.Valid = TRUE)) THEN
			AxisStep := STATE_ERROR_RESET;
		ELSE
			AxisStep := STATE_WAIT;
		END_IF
	END_IF

	(* central monitoring OF stop command attains a shorter reaction TIME in CASE OF emergency stop *)
	(******************CHECK FOR STOP COMMAND************************)
	IF (BasicControlAxis1.Command.Stop = TRUE) THEN
		IF ((AxisStep >= STATE_HOME) AND (AxisStep <= STATE_ERROR)) THEN
			(* reset all FB execute inputs we use *)
			MC_Home_0.Execute := 0;
			MC_Stop_0.Execute := 0;
			MC_MoveAbsolute_0.Execute := 0;
			MC_MoveAdditive_0.Execute := 0;
			MC_MoveVelocity_0.Execute := 0;
			MC_ReadAxisError_0.Acknowledge := 0;
			MC_Reset_0.Execute := 0;
			MC_Halt_0.Execute := 0;

			(* reset user commands *)
			BasicControlAxis1.Command.Halt := 0;
			BasicControlAxis1.Command.Home := 0;
			BasicControlAxis1.Command.MoveJogPos := 0;
			BasicControlAxis1.Command.MoveJogNeg := 0;
			BasicControlAxis1.Command.MoveVelocity := 0;
			BasicControlAxis1.Command.MoveAbsolute := 0;
			BasicControlAxis1.Command.MoveAdditive := 0;
			AxisStep := STATE_STOP;
		END_IF
	END_IF

	CASE AxisStep OF

		(******************** WAIT *************************)
		STATE_WAIT:  (* STATE: Wait *)
			IF (BasicControlAxis1.Command.Power = TRUE) THEN
				AxisStep := STATE_POWER_ON;
			ELSE
				MC_Power_0.Enable := FALSE;
			END_IF

			(* reset all FB execute inputs we use *)
			MC_Home_0.Execute := FALSE;
			MC_Stop_0.Execute := FALSE;
			MC_MoveAbsolute_0.Execute := FALSE;
			MC_MoveAdditive_0.Execute := FALSE;
			MC_MoveVelocity_0.Execute := FALSE;
			MC_Halt_0.Execute := FALSE;
			MC_ReadAxisError_0.Acknowledge := FALSE;
			MC_Reset_0.Execute := FALSE;

			(* reset user commands *)
			BasicControlAxis1.Command.Stop := FALSE;
			BasicControlAxis1.Command.Halt := FALSE;
			BasicControlAxis1.Command.Home := FALSE;
			BasicControlAxis1.Command.MoveJogPos := FALSE;
			BasicControlAxis1.Command.MoveJogNeg := FALSE;
			BasicControlAxis1.Command.MoveVelocity := FALSE;
			BasicControlAxis1.Command.MoveAbsolute := FALSE;
			BasicControlAxis1.Command.MoveAdditive := FALSE;

			BasicControlAxis1.Status.ErrorID := 0;

			(******************** POWER ON **********************)
		STATE_POWER_ON:  (* STATE: Power on *)
			MC_Power_0.Enable := TRUE;
			IF (MC_Power_0.Status = TRUE) THEN
				AxisStep := STATE_READY;
			END_IF
			(* if a power error occured go to error state *)
			IF (MC_Power_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_Power_0.ErrorID;
				AxisStep := STATE_ERROR;
			END_IF

			(******************** READY **********************)
		STATE_READY:  (* STATE: Waiting for commands *)
			IF (BasicControlAxis1.Command.Home = TRUE)THEN
				BasicControlAxis1.Command.Home := FALSE;
				AxisStep := STATE_HOME;

			ELSIF (BasicControlAxis1.Command.Stop = TRUE) THEN
				
				AxisStep := STATE_STOP;

			ELSIF (BasicControlAxis1.Command.MoveJogPos = TRUE) THEN
				AxisStep := STATE_JOG_POSITIVE;

			ELSIF (BasicControlAxis1.Command.MoveJogNeg = TRUE) THEN
				AxisStep := STATE_JOG_NEGATIVE;

			ELSIF (BasicControlAxis1.Command.MoveAbsolute = TRUE) THEN
				BasicControlAxis1.Command.MoveAbsolute := FALSE;
				AxisStep := STATE_MOVE_ABSOLUTE;

			ELSIF (BasicControlAxis1.Command.MoveAdditive = TRUE) THEN
				BasicControlAxis1.Command.MoveAdditive := FALSE;
				AxisStep := STATE_MOVE_ADDITIVE;

			ELSIF (BasicControlAxis1.Command.MoveVelocity = TRUE) THEN
				BasicControlAxis1.Command.MoveVelocity := FALSE;
				AxisStep := STATE_MOVE_VELOCITY;
            
			ELSIF (BasicControlAxis1.Command.Halt = TRUE) THEN
				BasicControlAxis1.Command.Halt := FALSE;
				AxisStep := STATE_HALT;                                    
			END_IF

			(******************** HOME **********************)
		STATE_HOME:  (* STATE: start homing process *)
			MC_Home_0.Position :=0;// BasicControlAxis1.Parameter.HomePosition;
			MC_Home_0.HomingMode :=mcHOME_DIRECT;// BasicControlAxis1.Parameter.HomeMode;
			MC_Home_0.Execute := TRUE;
			IF (MC_Home_0.Done = TRUE) THEN
				MC_Home_0.Execute := FALSE;
				AxisStep := STATE_READY;
				
			END_IF
			(* if a homing error occured go to error state *)
			IF (MC_Home_0.Error = TRUE) THEN
				MC_Home_0.Execute := FALSE;
				BasicControlAxis1.Status.ErrorID := MC_Home_0.ErrorID;
				AxisStep := STATE_ERROR;
			END_IF
        
			(***********************HALT_MOVEMENT***************************)
		STATE_HALT:   (* STATE: Halt movement *)
			MC_Halt_0.Deceleration := BasicControlAxis1.Parameter.Deceleration;
			MC_Halt_0.Execute := TRUE;
			IF (MC_Halt_0.Done = TRUE) THEN
				MC_Halt_0.Execute := FALSE;
				AxisStep := STATE_READY;        
			END_IF
			(* check if error occured *)
			IF (MC_Halt_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_Halt_0.ErrorID;
				MC_Halt_0.Execute := FALSE;
				AxisStep := STATE_ERROR;
			END_IF                                         

			(*********************** STOP MOVEMENT *************************)
		STATE_STOP:  (* STATE: Stop movement *)
			MC_Stop_0.Deceleration := BasicControlAxis1.Parameter.Deceleration;		
			MC_Stop_0.Execute := TRUE;
			(* if axis is stopped go to ready state *)
			IF ((MC_Stop_0.Done = TRUE) AND (BasicControlAxis1.Command.Stop = FALSE)) THEN
				MC_Stop_0.Execute := FALSE;
				AxisStep := STATE_READY;
			END_IF
			(* check if error occured *)
			IF (MC_Stop_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_Stop_0.ErrorID;
				MC_Stop_0.Execute := FALSE;
				AxisStep := STATE_ERROR;
			END_IF

			(******************** START JOG MOVEMENT POSITVE **********************)
		STATE_JOG_POSITIVE:  (* STATE: Start jog movement in positive direction *)
			MC_MoveVelocity_0.Velocity      := BasicControlAxis1.Parameter.Velocity;
			MC_MoveVelocity_0.Acceleration  := BasicControlAxis1.Parameter.Acceleration;
			MC_MoveVelocity_0.Deceleration  := BasicControlAxis1.Parameter.Deceleration;
			MC_MoveVelocity_0.Direction     := mcPOSITIVE_DIR;
			MC_MoveVelocity_0.Execute := TRUE;
			IF (BasicControlAxis1.Command.MoveJogPos = FALSE) THEN
				MC_MoveVelocity_0.Execute := FALSE;
				AxisStep := STATE_HALT;
			END_IF
			(* check if error occured *)
			IF (MC_MoveVelocity_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_MoveVelocity_0.ErrorID;
				MC_MoveVelocity_0.Execute := FALSE;
				AxisStep := STATE_ERROR;
			END_IF

			(******************** START JOG MOVEMENT NEGATIVE **********************)
		STATE_JOG_NEGATIVE:  (* STATE: Start jog movement in negative direction *)
			MC_MoveVelocity_0.Velocity      := BasicControlAxis1.Parameter.Velocity;
			MC_MoveVelocity_0.Acceleration  := BasicControlAxis1.Parameter.Acceleration;
			MC_MoveVelocity_0.Deceleration  := BasicControlAxis1.Parameter.Deceleration;
			MC_MoveVelocity_0.Direction	    := mcNEGATIVE_DIR;
			MC_MoveVelocity_0.Execute := TRUE;
			IF (BasicControlAxis1.Command.MoveJogNeg = FALSE) THEN
				MC_MoveVelocity_0.Execute := FALSE;
				AxisStep := STATE_HALT;
			END_IF
			(* check if error occured *)
			IF (MC_MoveVelocity_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_MoveVelocity_0.ErrorID;
				MC_MoveVelocity_0.Execute := FALSE;
				AxisStep := STATE_ERROR;
			END_IF

			(******************** START ABSOLUTE MOVEMENT **********************)
		STATE_MOVE_ABSOLUTE:  (* STATE: Start absolute movement *)
			MC_MoveAbsolute_0.Position      := BasicControlAxis1.Parameter.Position; 
			MC_MoveAbsolute_0.Velocity      := BasicControlAxis1.Parameter.Velocity;
			MC_MoveAbsolute_0.Acceleration  := BasicControlAxis1.Parameter.Acceleration;
			MC_MoveAbsolute_0.Deceleration  := BasicControlAxis1.Parameter.Deceleration;
			MC_MoveAbsolute_0.Direction     := BasicControlAxis1.Parameter.Direction;
			MC_MoveAbsolute_0.Execute := TRUE;
			(* check if commanded position is reached *)
			IF (BasicControlAxis1.Command.Halt) THEN
				BasicControlAxis1.Command.Halt := FALSE;
				MC_MoveAbsolute_0.Execute := FALSE;
				AxisStep := STATE_HALT;
			ELSIF (MC_MoveAbsolute_0.Done = TRUE) THEN
				MC_MoveAbsolute_0.Execute := FALSE;
				AxisStep := STATE_READY;
			END_IF
			(* check if error occured *)
			IF (MC_MoveAbsolute_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_MoveAbsolute_0.ErrorID;
				MC_MoveAbsolute_0.Execute := FALSE;
				AxisStep := STATE_ERROR;
			END_IF

			(******************** START ADDITIVE MOVEMENT **********************)
		STATE_MOVE_ADDITIVE:  (* STATE: Start additive movement *)
			MC_MoveAdditive_0.Distance      := BasicControlAxis1.Parameter.Position;
			MC_MoveAdditive_0.Velocity      := BasicControlAxis1.Parameter.Velocity;
			MC_MoveAdditive_0.Acceleration  := BasicControlAxis1.Parameter.Acceleration;
			MC_MoveAdditive_0.Deceleration  := BasicControlAxis1.Parameter.Deceleration;
			MC_MoveAdditive_0.Execute := TRUE;
			(* check if commanded distance is reached *)
			IF (BasicControlAxis1.Command.Halt) THEN
				BasicControlAxis1.Command.Halt := FALSE;
				MC_MoveAdditive_0.Execute := FALSE;
				AxisStep := STATE_HALT;
			ELSIF (MC_MoveAdditive_0.Done = TRUE) THEN
				MC_MoveAdditive_0.Execute := FALSE;
				AxisStep := STATE_READY;
			END_IF
			(* check if error occured *)
			IF (MC_MoveAdditive_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_MoveAdditive_0.ErrorID;
				MC_MoveAdditive_0.Execute := FALSE;
				AxisStep := STATE_ERROR;
			END_IF

			(******************** START VELOCITY MOVEMENT **********************)
		STATE_MOVE_VELOCITY:  (* STATE: Start velocity movement *)
			MC_MoveVelocity_0.Velocity      := BasicControlAxis1.Parameter.Velocity;
			MC_MoveVelocity_0.Acceleration  := BasicControlAxis1.Parameter.Acceleration;
			MC_MoveVelocity_0.Deceleration  := BasicControlAxis1.Parameter.Deceleration;
			MC_MoveVelocity_0.Direction     := BasicControlAxis1.Parameter.Direction;
			MC_MoveVelocity_0.Execute := TRUE;
			(* check if commanded velocity is reached *)
			IF (BasicControlAxis1.Command.Halt) THEN
				BasicControlAxis1.Command.Halt := FALSE;
				MC_MoveVelocity_0.Execute := FALSE;
				AxisStep := STATE_HALT;
			ELSIF (MC_MoveVelocity_0.InVelocity = TRUE) THEN
				MC_MoveVelocity_0.Execute := FALSE;
				AxisStep := STATE_READY;
			END_IF
			(* check if error occured *)
			IF (MC_MoveVelocity_0.Error = TRUE) THEN
				BasicControlAxis1.Status.ErrorID := MC_MoveVelocity_0.ErrorID;
				MC_MoveVelocity_0.Execute := FALSE;
				AxisStep := STATE_ERROR;
			END_IF

			(******************** FB-ERROR OCCURED *************************)
		STATE_ERROR:  (* STATE: Error *)
			(* check if FB indicates an axis error *)
			IF (MC_ReadAxisError_0.AxisErrorCount<>0) THEN
				AxisStep := STATE_ERROR_AXIS;
			ELSE
				IF (BasicControlAxis1.Command.ErrorAcknowledge = TRUE) THEN
					BasicControlAxis1.Command.ErrorAcknowledge := FALSE;
					BasicControlAxis1.Status.ErrorID := 0;
					(* reset axis if it is in axis state ErrorStop *)
					IF ((MC_ReadStatus_0.Errorstop = TRUE) AND (MC_ReadStatus_0.Valid = TRUE)) THEN
						AxisStep := STATE_ERROR_RESET;
					ELSE
						AxisStep := STATE_WAIT;
					END_IF
				END_IF
			END_IF

			(******************** AXIS-ERROR OCCURED *************************)
		STATE_ERROR_AXIS:  (* STATE: Axis Error *)
			IF (MC_ReadAxisError_0.Valid = TRUE) THEN
				IF (MC_ReadAxisError_0.AxisErrorID <> 0) THEN
					BasicControlAxis1.Status.ErrorID := MC_ReadAxisError_0.AxisErrorID;
				END_IF
				MC_ReadAxisError_0.Acknowledge := FALSE;
				IF (BasicControlAxis1.Command.ErrorAcknowledge = TRUE) THEN
					BasicControlAxis1.Command.ErrorAcknowledge := FALSE;
					(* acknowledge axis error *)
					IF (MC_ReadAxisError_0.AxisErrorID <> 0) THEN
						MC_ReadAxisError_0.Acknowledge := TRUE;
					END_IF
				END_IF
				IF (MC_ReadAxisError_0.AxisErrorCount = 0) THEN
					(* reset axis if it is in axis state ErrorStop *)
					BasicControlAxis1.Status.ErrorID := 0;
					IF ((MC_ReadStatus_0.Errorstop = TRUE) AND (MC_ReadStatus_0.Valid = TRUE)) THEN
						AxisStep := STATE_ERROR_RESET;
					ELSE
						AxisStep := STATE_WAIT;
					END_IF
				END_IF
			END_IF

			(******************** RESET DONE *************************)
		STATE_ERROR_RESET:  (* STATE: Wait for reset done *)
			MC_Reset_0.Execute := TRUE;
			(* reset MC_Power.Enable if this FB is in Error*)
			IF (MC_Power_0.Error = TRUE) THEN
				MC_Power_0.Enable := FALSE;
			END_IF
			IF(MC_Reset_0.Done = TRUE)THEN
				MC_Reset_0.Execute := FALSE;
				AxisStep := STATE_WAIT;
			ELSIF(MC_Reset_0.Error = TRUE) THEN
				MC_Reset_0.Execute := FALSE;
				AxisStep := STATE_ERROR;                    
			END_IF
		(******************** SEQUENCE END *************************)
	END_CASE

(***************************************************************
        Function Block Calls
***************************************************************)

(************************** MC_POWER ****************************)
	MC_Power_0.Axis := Axis1Obj;  (* pointer to axis *)
	MC_Power_0();

	(************************** MC_HOME *****************************)
	MC_Home_0.Axis := Axis1Obj;
	MC_Home_0();

	(********************** MC_MOVEABSOLUTE *************************)
	MC_MoveAbsolute_0.Axis := Axis1Obj;
	MC_MoveAbsolute_0();
	(********************** MC_MOVEADDITIVE *************************)
	MC_MoveAdditive_0.Axis := Axis1Obj;
	MC_MoveAdditive_0();

	(********************** MC_MOVEVELOCITY *************************)
	MC_MoveVelocity_0.Axis := Axis1Obj;
	MC_MoveVelocity_0();

	(************************** MC_STOP *****************************)
	MC_Stop_0.Axis := Axis1Obj;
	MC_Stop_0();

	(***************************MC_HALT******************************)
	MC_Halt_0.Axis := Axis1Obj;
	MC_Halt_0();

	(************************** MC_RESET ****************************)
	MC_Reset_0.Axis := Axis1Obj;
	MC_Reset_0();

	
	typDriveLatchPos.Continuous2:=TRUE;
	typDriveLatchPos.UsePosEdge2:=TRUE;

	
	MC_BR_InitModPos_0.Axis:=Axis1Obj;
	MC_BR_InitModPos_0.Period:=AxisPeriod;
	MC_BR_InitModPos_0.Factor:=1;
	MC_BR_InitModPos_0();	
	
	
		
	
	IF (BasicControlAxis1.Command.ErrorAcknowledge = TRUE) THEN
		gSdcAxisCtrl.cmd.ackError[1]:=TRUE;
	END_IF
	

	
	
END_PROGRAM

